/*
  gps_flash.c  Erase the flash memory that I use 
  Jed Margolin 1/17/2025
  board: pico-gps-r4   Can be powered by +9V or by USB power, the choice is made when the board is stuffed
  Use 4-pin Blue NEO7M GPS module (not used here)
  Use bme280 for temperature, humidity, and air pressure (not used here)
  
  Uses the keyboard

  uart0 can use the ch340 (uart to USB adapter) with the AccessPort program for Windows (https://accessport.soft32.com).
*/

#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "hardware/uart.h"
#include "hardware/structs/i2c.h"
#include "pico/binary_info.h"
#include <math.h>
#include "main.h"          // mine

#define FLAG1 0     // For testing
//================================================================
// Used to read and debounce the switches during settings mode
bool repeating_timer_callback(struct repeating_timer *t) 
{
   irq_flag = 1;

   // read switches
   do_switch(PB1, &switch1);   // PB1, switch structure  INC
   do_switch(PB2, &switch2);   // PB2, switch structure  DEC
   do_switch(PB3, &switch3);   // PB3, switch structure  MODE
   
   irq_count--;
   if(irq_count == 0)  
   { 
      irq_count = 250;   // 250 = 1 second
      secs_flag = 1;      // seconds flag
   }

   return (true);
}
//===============================================================
int main () 
{
   uint16_t n;

// Hardware on the board
   gpio_init(LED1);      // Green
   gpio_set_dir(LED1, GPIO_OUT);
   gpio_put(LED1,0);

   gpio_init(LED2);      // Red
   gpio_set_dir(LED2, GPIO_OUT);
   gpio_put(LED2,0);

   gpio_init(PB1);       // Pushbutton 1
   gpio_set_dir(PB1, GPIO_IN);
   gpio_pull_up(PB1);

   gpio_init(PB2);       // Pushbutton 2
   gpio_set_dir(PB2, GPIO_IN);
   gpio_pull_up(PB2);

   gpio_init(PB3);       // Pushbutton 3
   gpio_set_dir(PB3, GPIO_IN);
   gpio_pull_up(PB3);

   gpio_init(SW);        // Slide Switch
   gpio_set_dir(SW, GPIO_IN);
   gpio_pull_up(SW);

   gpio_init(CHAN);      // Header Pin, make it an output for testing
   gpio_set_dir(CHAN, GPIO_OUT);
   gpio_put(CHAN,0);
//-----------------------------------
// For the bme-280 where used
// This uses I2C0 on the default SDA and SCL pins (4, 5)
   i2c_init(i2c_default, 300 * 1000);   // 300 KHz (400 KHz nax)
   gpio_set_function(PICO_DEFAULT_I2C_SDA_PIN, GPIO_FUNC_I2C);
   gpio_set_function(PICO_DEFAULT_I2C_SCL_PIN, GPIO_FUNC_I2C);
   gpio_pull_up(PICO_DEFAULT_I2C_SDA_PIN);
   gpio_pull_up(PICO_DEFAULT_I2C_SCL_PIN);
//    bi_decl(bi_2pins_with_func(PICO_DEFAULT_I2C_SDA_PIN, PICO_DEFAULT_I2C_SCL_PIN, GPIO_FUNC_I2C));
//-----------------------------------
// Set up the UARTs with the required speed.
   uart_init(UART_ID, BAUD_RATE);

// Set the TX and RX pins by using the function select on the GPIO
   gpio_set_function(UART_TX_PIN, GPIO_FUNC_UART);
   gpio_set_function(UART_RX_PIN, GPIO_FUNC_UART);

// Send the title message and version out through uart
   uart_puts(UART_ID, version);   // send it out
   
// uart for GPS
// Set up the UART with the required speed.
   uart_init(UART_ID1, BAUD_RATE1);

// Set the TX and RX pins by using the function select on the GPIO
   gpio_set_function(UART_TX1_PIN, GPIO_FUNC_UART);
   gpio_set_function(UART_RX1_PIN, GPIO_FUNC_UART);
//----------------------------
   lcd_init();

// Greetings to LCD  - Canned Messages in Flash
   domsg(&dmsg1);    // "Jed Margolin's"
   domsg(&dmsg2);    // "GPS Flash"

   sleep_ms(3000);   // 3 seconds

   SetCmdMode();     // set for commands
   lcd_byte(0x01);   // clear lcd, both lines

   domsg(&dmsg3);    // "[date]"
//------------------------------------
   blink_led(200, 1);
//============================================
// uart0: make sure uart FIFO is empty, disable FIFO for this (keyboard and monitor)
   uart_set_fifo_enabled(UART_ID, false);
   for(n=0; n<64; n++)   
   {
       // keep going until FIFO is empty (0 bytes in FIFO)
       if( uart_is_readable(UART_ID) )   { uart_char = uart_getc(UART_ID); }
       else {break;}
   }
//   uart_set_fifo_enabled(UART_ID, true);    // do not use FIFO

// uart1: make sure uart FIFO is empty, Use FIFO for this (GPS module)
   uart_set_fifo_enabled(UART_ID1, false);
   for(n=0; n<64; n++)   
   {
       // keep going until FIFO is empty (0 bytes in FIFO)
       if( uart_is_readable(UART_ID1) )   { uart_char = uart_getc(UART_ID1); }
       else {break;}
   }
   uart_set_fifo_enabled(UART_ID1, true);  // uart FIFO for GPS

   // Does this work? It should cause the RX FIFO to trigger an interrupt when the FIFO has 28 bytes
   uart_get_hw(UART_ID1)->ifls=0x04;  
//=========================================
   irq_flag = 0;
   irq_count = 250;
   secs_flag = 0;
   erase_flag = 0;

   // Create a repeating timer that calls repeating_timer_callback
   // If the delay is negative the next callback will be starting from the beginning of the callback,
   // Positive starts from the end of the callback
   add_repeating_timer_ms(-4,repeating_timer_callback, NULL, &timer);  // 4 ms
//----------------------------
   blink_led(500, 2);

/*
     erase_log_sectors();
     myflash_erase_sector(FLASH_SDATE_SECTOR);
     myflash_erase_sector(FLASH_BREF_SECTOR);
     myflash_erase_sector(FLASH_LOCAL_SECTOR);
     myflash_erase_sector(FLASH_DST_SECTOR);

     LED_break();
*/
//==============================================
// GPS main
   sleep_ms(3000);

   SetCmdMode();     // set for commands
   lcd_byte(0x01);   // clear lcd, both lines
   domsg(&dmsg11);   // "Ready"

   uart_puts(UART_ID, "Keyboard Commands (Upper or Lower Case)\n");
   uart_puts(UART_ID, "   V: Show Version\n");   
   uart_puts(UART_ID, "   D: Dump All of the Flash Logs to uart\n");
   uart_puts(UART_ID, "   1: Erase Data Flash\n");
   uart_puts(UART_ID, "   2: Erase Start Date Flash\n");
   uart_puts(UART_ID, "   3: Erase Barometric Reference Flash\n");
   uart_puts(UART_ID, "   4: Erase Local Offset Flash\n");
   uart_puts(UART_ID, "   5: Erase DST Dates Flash\n");
   uart_puts(UART_ID, "   H: Repeat This List\n\n"); 

   uart_puts(UART_ID, "> Ready For Commands\n\n"); 

   while(1)
   {
      do_uart_RX();    // Use uart RX to decide what to send to uart
   }

} // end main
//================================================================
// Use uart RX to decide what to do
void do_uart_RX()
{
    if( uart_is_readable(UART_ID) )   
   { 

      uart_char = uart_getc(UART_ID) & (~0x20);    // read uart FIFO (upper or lower case)

      switch(uart_char)
      {

         case 'V':   // Version message
            strcpy(line, version);
            uart_puts(UART_ID, line); 
            break;
          //----------
          case 'D':    // Dump First 32 pages of the Flash logs to uart
            uart_puts(UART_ID,"\nFlash Log 32 pages\n");      
            dump_flash(0, 32);   // (erase_flag, # of pages)
            break;
          //----------
         case ('1' & (~0x20) ):    // Erase Flash Data
            uart_puts(UART_ID, "\nErase Data Flash?   Y/N\n"); 
            erase_flag = 1;
            break;
          //----------   
         case ('2' & (~0x20) ):    // Erase Start Date Flash Sector
            dump_flash_page_hex(FLASH_SDATE_SECTOR);
            uart_puts(UART_ID, "\nErase Start Date Sector?   Y/N\n"); 
            erase_flag = 2;
            break;
          //----------   
         case ('3' & (~0x20) ):    // Erase Barometric Reference Flash Sector
            dump_flash_page_hex(FLASH_BREF_SECTOR);
            uart_puts(UART_ID, "\nErase Barometric Reference?   Y/N\n"); 
            erase_flag = 3;
            break;
          //----------   
         case ('4' & (~0x20) ):    // Erase Local Offset Flash Sector
            dump_flash_page_hex(FLASH_LOCAL_SECTOR);
            uart_puts(UART_ID, "\nErase Local Offset Flash?   Y/N\n"); 
            erase_flag = 4;
            break;
          //----------   
         case ('5' & (~0x20) ):    // Erase DST Date Flash Sector
            dump_flash_page_hex(FLASH_DST_SECTOR);
            uart_puts(UART_ID, "\nErase DST Dates Flash?   Y/N\n"); 
            erase_flag = 5;
            break;
         //----------
         case 'Y':    // Yes
            if(erase_flag == 1)  
            {
               erase_log_sectors();
               uart_puts(UART_ID, "\nFlash Log Data Erased\n\n"); 
 
               SetCmdMode();     // set for commands
               lcd_byte(0x01);   // clear lcd, both lines
               domsg(&dmsg5);
               domsg(&dmsg4);

               erase_flag = 0;    
               break;
            }

            if(erase_flag == 2)  
            {
               myflash_erase_sector(FLASH_SDATE_SECTOR);
               uart_puts(UART_ID, "\nFlash SDATE Data Erased\n\n"); 

               SetCmdMode();     // set for commands
               lcd_byte(0x01);   // clear lcd, both lines
               domsg(&dmsg6);
               domsg(&dmsg4);

               erase_flag = 0;
               break;
            }

           if(erase_flag == 3)  
           {
              myflash_erase_sector(FLASH_BREF_SECTOR);
              uart_puts(UART_ID, "\nFlash Barometric Reference Data Erased\n\n"); 
 
              SetCmdMode();     // set for commands
              lcd_byte(0x01);   // clear lcd, both lines
              domsg(&dmsg7);
              domsg(&dmsg4);

              erase_flag = 0;
              break;
           }

           if(erase_flag == 4)  
           {
              myflash_erase_sector(FLASH_LOCAL_SECTOR);
              uart_puts(UART_ID, "\nFlash Local Offset Data Erased\n\n"); 
              SetCmdMode();     // set for commands
              lcd_byte(0x01);   // clear lcd, both lines
              domsg(&dmsg8);
              domsg(&dmsg4);

              erase_flag = 0;
              break;
           }

           if(erase_flag == 5)  
           {
              myflash_erase_sector(FLASH_DST_SECTOR);
              uart_puts(UART_ID, "\nFlash DST Data Erased\n\n"); 
              SetCmdMode();     // set for commands
              lcd_byte(0x01);   // clear lcd, both lines
              domsg(&dmsg9);
              domsg(&dmsg4);

              erase_flag = 0;
              break;
           }
         //----------
         case 'N':    // No
            erase_flag = 0;
            uart_puts(UART_ID, "\nFlash Data Not Erased\n\n"); 

            SetCmdMode();     // set for commands
            lcd_byte(0x01);   // clear lcd, both lines
            domsg(&dmsg10);

            break;
         //----------
         case 'H':    // Repeat this List
            uart_puts(UART_ID, "Keyboard Commands (Upper or Lower Case)\n");
            uart_puts(UART_ID, "   V: Show Version\n");   
            uart_puts(UART_ID, "   D: Dump All of the Flash Logs to uart\n");
            uart_puts(UART_ID, "   1: Erase Data Flash\n");
            uart_puts(UART_ID, "   2: Erase Start Date Flash\n");
            uart_puts(UART_ID, "   3: Erase Barometric Reference Flash\n");
            uart_puts(UART_ID, "   4: Erase Local Offset Flash\n");
            uart_puts(UART_ID, "   5: Erase DST Dates Flash\n");
            uart_puts(UART_ID, "   H: Repeat This List\n\n"); 
            break;
         //----------
         default:
            erase_flag = 0;
 
      }  // end switch
   }  // end of if uart was readable
} // end uart RX
//====================================================
//====================================================
void blink_led(uint16_t time, uint16_t times)
{
    unsigned int n;

    for(n=0; n<times; n++)
    { 
       gpio_put(LED1,1);
       sleep_ms(time);
       gpio_put(LED1,0);
       sleep_ms(time);
    }
}
//=============================================
void do_switch(unsigned char bit, struct myswitch *sptr)    // Bit, switch structure
{
// Debounce and process the switch
// sptr points to the switch's data block
    sptr->debounce  = sptr->debounce <<1;    // shifts in a zero

    if( gpio_get(bit) )        // a zero has already been shifted in
    {
       sptr->debounce++ ;      // so make LSB a one
    }

// Debounced state is either 0x00 or 0xFF
    if(sptr->debounce == 0x00)  {sptr->state = 0x00;}
    if(sptr->debounce == 0xFF)  {sptr->state = 0x01;}

// process the switch
    if(sptr->state != sptr->old)     // we have a change in switch state
    {
       if( (sptr->old == 0x01) && (sptr->state == 0x00) )  // we have a switch press
       {
          sptr->press = 0x01;
       }

       if( (sptr->old == 0x00) && (sptr->state== 0x01) )  // we have a switch release
       {
          sptr->release = 0x01;
       }

       sptr->old = sptr->state;     // the new state is now the old state
    }

// We now know if a switch has been pressed and if a switch has been released
}
//==========================================
void test_sint16(int8_t n, int16_t var)
{
   uint8_bcd_str(n, line);  // unsigned char to ASCII string, 3 digits
   strcat(line," Test: ");
   sint16_bcd_str(var, nstr);  // int16 to ASCII string, 5 digits
   strcat(line,nstr);
   strcat(line,"\n");
   uart_puts(UART_ID, line);
}
//======================================================
void uint16_hex_bin_str(uint16_t value, char *str)
{
   #define decimal 5
   uint32_t factor;
   uint16_t remains;
   uint8_t bcd[8];
   uint8_t m, n;

//   value = 65535;
   
   remains = value;
   for(n=0; n<decimal; ++n)
   {
      factor = 1;
      for (m=0; m<(decimal - n - 1); ++m)
      factor *= 10;
      bcd[decimal - n - 1] = remains / factor;
      remains = remains % factor;
    }

   str[0] = bcd[4] + 0x30;
   str[1] = bcd[3] + 0x30;
   str[2] = bcd[2] + 0x30;
   str[3] = bcd[1] + 0x30;
   str[4] = bcd[0] + 0x30;
   str[5] = 0;

/*
   strcpy(line,"\nTest Hex to ASCII: ");
   strcat(line,nstr);
   strcat(line,"\n\n");
   uart_puts(UART_ID, line);   // send it out
*/
}
//================================================
